/**
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <math.h>

#include "ApproxMath.h"

using namespace Rebels;

static const constexpr int MAX_APPROXIMATIONS = 360 * 4;
#define USE_APPROX_TRIGONOMETRY

double ApproxMath::Sin(units::radian_t angle) {
#ifdef USE_APPROX_TRIGONOMETRY
  static double approximations[MAX_APPROXIMATIONS];
  static bool init = false;
  if (!init) {
    init = true;
    for (int i = 0; i < MAX_APPROXIMATIONS; ++i) {
      const double a = 360.0 / ((double)MAX_APPROXIMATIONS) * ((double)i);
      approximations[i] = std::sin(M_PI * a / 180.0);
    }
  }

  int i = ((double)MAX_APPROXIMATIONS) / 360.0 * (angle.convert<units::degree>().value());
  if (i < 0) i = 0;
  if (i >= MAX_APPROXIMATIONS) i = MAX_APPROXIMATIONS - 1;
  return approximations[i];
#else
  return std::sin(angle.value());
#endif
}

double ApproxMath::Cos(units::radian_t angle) {
#ifdef USE_APPROX_TRIGONOMETRY
  static double approximations[MAX_APPROXIMATIONS];
  static bool init = false;
  if (!init) {
    init = true;
    for (int i = 0; i < MAX_APPROXIMATIONS; ++i) {
      const double a = 360.0 / ((double)MAX_APPROXIMATIONS) * ((double)i);
      approximations[i] = std::cos(M_PI * a / 180.0);
    }
  }

  int i = ((double)MAX_APPROXIMATIONS) / 360.0 * (angle.convert<units::degree>().value());
  if (i < 0) i = 0;
  if (i >= MAX_APPROXIMATIONS) i = MAX_APPROXIMATIONS - 1;
  return approximations[i];
#else
  return std::cos(angle.value());
#endif
}
