/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <frc/TimedRobot.h>

class BaseRobot {
public:
  virtual void RobotInit() = 0;
  virtual void RobotPeriodic() = 0;
  virtual void EnabledInit() = 0;
  virtual void DisabledInit() = 0;
  virtual void DisabledPeriodic() = 0;
  virtual void AutonomousInit() = 0;
  virtual void AutonomousPeriodic() = 0;
  virtual void TeleopInit() = 0;
  virtual void TeleopPeriodic() = 0;

  units::second_t TimeSinceDisabled() const {
    return frc::Timer::GetFPGATimestamp() - internal.disableTime;
  }

  BaseRobot() : internal(this) {}
  virtual ~BaseRobot() = default;
  void StartCompetition() { internal.StartCompetition(); }
  void EndCompetition() { internal.EndCompetition(); }

private:
  class Internal : public frc::TimedRobot {
    friend class BaseRobot;

  public:
    explicit Internal(BaseRobot* parent) { this->parent = parent; }
    virtual ~Internal() = default;
    virtual void RobotInit() override;
    virtual void RobotPeriodic() override;
    virtual void EnabledInit();
    virtual void DisabledInit() override;
    virtual void DisabledPeriodic() override;
    virtual void AutonomousInit() override;
    virtual void AutonomousPeriodic() override;
    virtual void TeleopInit() override;
    virtual void TeleopPeriodic() override;

  private:
    BaseRobot* parent{ 0 };
    units::second_t disableTime{ 0.0 };
    bool wasEnabled{ false };
    bool wasInMatch{ false };
    bool enabledInitCalled{ false };
  };

  Internal internal;
};
