/**
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "BaseTeleop.h"
#include "Telemetry.h"

using namespace Rebels;

#define TELEMETRY_CATEGORY "Teleop"

void BaseTeleop::EnableDrive() {
  LOG_INFO(TELEMETRY_CATEGORY, "Taking over the drive system");
  if (drive.get() == nullptr) {
    drive.reset(new Drive(SystemName));
  }
  (*drive)->SetCoastMode();
}

void BaseTeleop::DisableDrive() {
  if (drive.get() != nullptr) {
    LOG_INFO(TELEMETRY_CATEGORY, "Releasing the drive system");
    drive.reset(nullptr);
  }
}

bool BaseTeleop::HasDrive() {
  return (drive.get() != nullptr && (*drive)->GetOwner() == SystemName);
}
