/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <frc/XboxController.h>

#include "Constants.h"
#include "Drive.h"

class BaseTeleop {
public:
  BaseTeleop() = default;
  ~BaseTeleop() = default;

  // Call to have Teleop take over the drive and other systems
  virtual void EnableDrive();
  virtual void DisableDrive();
  bool HasDrive();

  // Periodic is called every tick only when in teleop mode
  virtual void Periodic() = 0;

  // Returns true if the joystick is moved
  virtual bool IsJoystickMoved() = 0;

  // Clear out any pending button presses
  virtual void ClearButtonState() = 0;

  // Rumbler
  virtual void Rumble() = 0;

protected:
  static const constexpr std::string_view SystemName{ "Teleop" };
  std::unique_ptr<Rebels::Drive> drive;
};
