/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once
#include <tuple>
#include <frc/util/Color.h>

namespace Rebels {

  std::tuple<double, double, double> rgbToHsv(double r, double g, double b) {
    double max = std::max(std::max(r, g), b);
    double v = max;

    if (max == 0) {
      return { 0, 0, 0 };
    }

    double min = std::min(std::min(r, g), b);
    double delta = max - min;
    if (delta == 0) {
      return { 0, 0, v };
    }

    double h;
    double s = (max - min) / max;

    if (max == r) {
      h = 60 * ((g - b) / delta);
    }
    else if (max == g) {
      h = 60 * ((b - r) / delta) + 120;
    }
    else {
      h = 60 * ((r - g) / delta) + 240;
    }

    if (h < 0) {
      h += 360;
    }

    return { h, s, v };
  }


  class Color : public frc::Color {
  public:
    Color(frc::Color color) : frc::Color(color.red, color.green, color.blue) {
      auto [h, s, v] = rgbToHsv(color.red, color.green, color.blue);
      hue = h;
      saturation = s;
      value = v;
    }

    double hue = 0.0;
    double saturation = 0.0;
    double value = 0.0;
  };

} // namespace Rebels
