/**
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <frc/Timer.h>

#include "Compressor.h"
#include "Settings.h"
#include "StateMachines.h"
#include "Telemetry.h"

using namespace Rebels;

#define TELEMETRY_CATEGORY "Compressor"

Compressor::Compressor() : Updateable("Compressor") {
  Rebels::StateMachines::Register(this);
}

Compressor::~Compressor() {
  Rebels::StateMachines::Unregister(this);
}

void Compressor::BackgroundUpdate() {

#ifdef ENABLE_COMPRESSOR
  // Check and make sure that the compressor isn't going full time and burning out.
  const auto now = frc::Timer::GetFPGATimestamp();
  if (lastCheck < 1_ms) {
    lastCheck = now;
  }

  const units::second_t period = now - lastCheck;
  lastCheck = now;

  const auto current = compressor.GetCurrent();
  LOG_VALUE(TELEMETRY_CATEGORY, "Current (A)", current.value());

  if (current > 2_A) {
    runningTime += period;
    if (runningTime > maxRunningTime) {
      LOG_ERROR(TELEMETRY_CATEGORY, fmt::format("Exceeded allowable run time of {} seconds [!]", runningTime.value()));
      compressor.Disable();
    }
  }

  LOG_VALUE(TELEMETRY_CATEGORY, "Runtime (sec)", runningTime.value());
#endif
}
