/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2020 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once
#include <stdint.h>
#include <atomic>
#include <mutex>
#include <vector>
#include <frc/AddressableLED.h>

#include "Updateable.h"

typedef std::vector<frc::AddressableLED::LEDData> LedVector;

struct LedPattern {
  LedPattern(const char* patternName) : name(patternName) { running = false; }
  virtual ~LedPattern() = default;
  virtual bool Update(LedVector& data) = 0;
  std::string name;
  std::atomic<bool> running;
  int length{ 1 };
  int count{ 0 };
  int delayCount{ 1 };
  frc::Color color;
};

class Led : public Rebels::Updateable {
public:
  Led(int pwmId, int length);
  virtual ~Led();

  void StartAroundTheWorld(frc::Color color, units::time::millisecond_t delay);
  void StartFlasher(frc::Color color, units::time::millisecond_t delay);
  void StartDanceParty(units::time::millisecond_t delay);
  void SetMeter(frc::Color mainColor, frc::Color secondaryColor, double percent);
  void Stop();

protected:
  virtual void Update() override;

private:
  frc::AddressableLED leds;
  LedVector data;
  int length{ 0 };

  std::mutex patternMutex;
  LedPattern* pattern{ nullptr };
};
