/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2020 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <cstddef>

template <class T, int N>
class MovingAverage {
public:
  MovingAverage() = default;
  ~MovingAverage() = default;

  void Add(T d) {
    if (index >= N) {
      index = 0;
    }
    values[index] = d;
    index++;
    if (count < N) {
      count++;
    }
  }

  double Average() const {
    if (count == 0) {
      return 0;
    }

    double total = 0;
    for (size_t i = 0; i < count; i++) {
      total += values[i];
    }
    return total / count;
  }

  void Reset() {
    index = 0;
    count = 0;
  }

private:
  T values[N];
  size_t index = 0;
  size_t count = 0;
};
