/**
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "Periodic.h"

using namespace Rebels;

Periodic::Periodic() {}

Periodic::~Periodic() { StopPeriodic(); }

bool Periodic::IsPeriodicRunning() const { return running; }
bool Periodic::IsPeriodicStopped() const { return !running; }

void Periodic::StartPeriodic(const units::second_t period) {
  notifier.reset(new frc::Notifier(std::bind(NotifierFn, this)));
  notifier->StartPeriodic(period);
  running = true;
}

void Periodic::StopPeriodic() {
  running = false;
  notifier->Stop();
  notifier.reset(nullptr);
}

double Periodic::RandomMilliseconds() const {
  // Generate a small random number of ms to return
  return (rand() % 10) / 100.0;
}

void Periodic::NotifierFn(Periodic* pThis) {
  if (pThis) {
    pThis->Tick();
  }
}
