/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <frc/Notifier.h>

namespace Rebels {

  class Periodic {
  public:
    Periodic();
    virtual ~Periodic();

    void StartPeriodic(const units::second_t period = 20_ms);
    void StopPeriodic();
    bool IsPeriodicRunning() const;
    bool IsPeriodicStopped() const;

    double RandomMilliseconds() const;

  protected:
    virtual void Tick() = 0;

  private:
    static void NotifierFn(Periodic* pThis);
    bool running{ false };
    std::unique_ptr<frc::Notifier> notifier;
  };

} // namespace Rebels
