/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2019 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "Settings.h"
#include "Singleton.h"
#include "Updateable.h"

namespace Rebels {

  class PiBroadcaster : public Updateable {
    DECLARE_SINGLETON(PiBroadcaster)

  protected:
    virtual void Update() override {}
    virtual void BackgroundUpdate() override;

  public:
    virtual ~PiBroadcaster() = default;

  private:
    PiBroadcaster();

  private:
    units::second_t lastBroadcast{ 0 };
    static const constexpr units::second_t broadcastPeriod{ 10 };
  };

} // namespace Rebels
