/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2020 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <frc/AnalogInput.h>
#include <frc/smartdashboard/SmartDashboard.h>
#include <units/pressure.h>
#include <units/voltage.h>

#include "StateMachines.h"
#include "Updateable.h"
#include "Telemetry.h"

namespace Rebels {

  class PressureSensor : public Rebels::Updateable {
  public:
    PressureSensor(int channel) : Updateable("PressureSensor"), sensorIn(channel) {
      Rebels::StateMachines::Register(this);
    }

    virtual ~PressureSensor() {
      Rebels::StateMachines::Unregister(this);
    }

    virtual void Update() override {}

    virtual void BackgroundUpdate() override {
      LOG_VALUE("Pressure Sensor", "Pressure (PSI)", GetPressure().value());
    }

    units::pounds_per_square_inch_t GetPressure() const {
      // product-specific voltage->pressure conversion, see product manual
      // in this case, 250(V/5)-25
      return units::pounds_per_square_inch_t(250.0 * (GetVoltage().value() / 5.0) - 25.0);
    }

    units::volt_t GetVoltage() const {
      return units::volt_t(sensorIn.GetVoltage());
    }

  private:
    frc::AnalogInput sensorIn;
  };

} // namespace Rebels
