/**
 *  ____ _____ ___ ____            ____      _          _
 * |___ \___  / _ \___ \          |  _ \ ___| |__   ___| |___
 *   __) | / / | | |__) |  _____  | |_) / _ \ '_ \ / _ \ / __|
 *  / __/ / /| |_| / __/  |_____| |  _ <  __/ |_) |  __/ \__ \
 * |_____/_/  \___/_____|         |_| \_\___|_.__/ \___|_|___/
 *
 *
 * Copyright 2020 Kitchener-Waterloo Youth Robotics
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include <atomic>
#include <shared_mutex>
#include <thread>
#include <vector>
#include <units/time.h>

#include "Singleton.h"
#include "Updateable.h"

namespace Rebels {

  class StateMachines {
    DECLARE_SINGLETON(StateMachines)

  public:
    virtual ~StateMachines() = default;

    static void UpdateAll() { Get()->UpdateAll_(); }
    static void Register(Updateable* sm) { Get()->Register_(sm); }
    static void Unregister(Updateable* sm) { Get()->Unregister_(sm); }
    static void UpdateOrder() { Get()->UpdateOrder_(); }

  private:
    StateMachines();
    void UpdateAll_();
    void Register_(Updateable* sm);
    void Unregister_(Updateable* sm);
    void UpdateOrder_();

  private:
    std::vector<Updateable*> stateMachines;
    std::shared_mutex mutex;
    std::atomic<bool> updateOrder{ false };

    // Low priority background thread.
    void BackgroundThread();
    std::thread backgroundThread;
    std::atomic<bool> backgroundShouldRun{ false };
    units::millisecond_t backgroundLoopDelay{ 250.0 };
  };

} // namespace Rebels
