#include "AutoBasic.h"

using namespace Rebels;

#define TELEMETRY_CATEGORY "AutoBasic"

REGISTER_DOUBLE(AutoBasicSteer, "Auto/Basic/Steer", 0)
REGISTER_DOUBLE(AutoBasicSpeed, "Auto/Basic/Speed", 0.7)
REGISTER_DOUBLE(AutoBasicDriveTime, "Auto/Basic/Drive Time", 1.5)

AutoBasic::~AutoBasic() {
  ReleaseDrive();
}

void AutoBasic::Periodic() {
  // let all the various state machines have a chance to wind up
  if (warmupTicks++ < 2) {
    return;
  }

  if (done) {
    return;
  }

  if (driving) {
    (*drive)->ArcadeDrive(GetAutoBasicSteer(), GetAutoBasicSpeed(), true);

    const auto timestamp = frc::Timer::GetFPGATimestamp();
    if (driveStartedAt + units::second_t(GetAutoBasicDriveTime()) < timestamp) {
      Stop();
    }
  }
  else {
    Go();
  }
}

void AutoBasic::Go() {
  LOG_INFO(TELEMETRY_CATEGORY, "Auto starting");
  drive.reset(new Drive(SystemName));

  driving = true;
  driveStartedAt = frc::Timer::GetFPGATimestamp();

  (*drive)->ShiftHigh();
}

void AutoBasic::Stop() {
  LOG_INFO(TELEMETRY_CATEGORY, "Auto stopping");
  driving = false;
  done = true;

  if (drive.get() != nullptr) {
    (*drive)->Stop();
  }

  ReleaseDrive();
}

void AutoBasic::ReleaseDrive() {
  if (drive.get() != nullptr) {
    drive.reset(nullptr);
  }
}
