#include "AutoShoot.h"

using namespace Rebels;

#define TELEMETRY_CATEGORY "AutoShoot"

REGISTER_DOUBLE(AutoShootSteer, "Auto/Shoot/Steer", 0)
REGISTER_DOUBLE(AutoShootSpeed, "Auto/Shoot/Speed", 0.7)
REGISTER_DOUBLE(AutoShootDriveTime, "Auto/Shoot/Drive Time", 1.5)
REGISTER_DOUBLE(AutoShootShootTime, "Auto/Shoot/Shoot Time", 5)

AutoShoot::~AutoShoot() {
  ReleaseDrive();
}

void AutoShoot::Periodic() {
  // let all the various state machines have a chance to wind up
  if (warmupTicks++ < 2) {
    return;
  }

  if (done) {
    return;
  }

  const auto timestamp = frc::Timer::GetFPGATimestamp();
  const auto pointGuard = PointGuard::Get();
  switch (autoState)
  {
  case IDLE:
    drive.reset(new Drive(SystemName));
    driveStartedAt = frc::Timer::GetFPGATimestamp();
    autoState = DRIVING;
    break;
  case DRIVING:
    (*drive)->ArcadeDrive(GetAutoShootSteer(), GetAutoShootSpeed(), true);
    if (driveStartedAt + units::second_t(GetAutoShootDriveTime()) <= timestamp) {
      autoState = START_SHOOTING;
      (*drive)->ArcadeDrive(0, 0, true);
    }
    break;
  case START_SHOOTING:
    shootStartedAt = frc::Timer::GetFPGATimestamp();
    (pointGuard)->StartShootingVision();
    autoState = SHOOTING;
    break;
  case SHOOTING:
    (*drive)->ArcadeDrive(0, 0, true);
    if (shootStartedAt + units::second_t(GetAutoShootShootTime()) <= timestamp) {
      (pointGuard)->StopShooting();
    }
    break;
  }
}

void AutoShoot::Go() {
  LOG_INFO(TELEMETRY_CATEGORY, "Auto starting");
  drive.reset(new Drive(SystemName));
  const auto pointGuard = PointGuard::Get();

  //(pointGuard)->StartShooting();
  shooting = true;
  shootStartedAt = frc::Timer::GetFPGATimestamp();

  (*drive)->ShiftHigh();
}

void AutoShoot::Stop() {
  LOG_INFO(TELEMETRY_CATEGORY, "Auto stopping");
  driving = false;
  done = true;

  if (drive.get() != nullptr) {
    (*drive)->Stop();
  }

  ReleaseDrive();
}

void AutoShoot::ReleaseDrive() {
  if (drive.get() != nullptr) {
    drive.reset(nullptr);
  }
}
