#include <frc/smartdashboard/SmartDashboard.h>

#include "Autonomous.h"
#include "Telemetry.h"
#include "Auto/AutoBasic.h"
#include "Auto/AutoShoot.h"

#define TELEMETRY_CATEGORY "Autonomous"

static const constexpr std::string_view AutoModeName = "Auto Mode";

Autonomous::Autonomous() {
  autoChooser.SetDefaultOption(AutoDoNothing, AutoDoNothing);
  autoChooser.AddOption(AutoModeBasic, AutoModeBasic);
  autoChooser.AddOption(AutoModeShoot, AutoModeShoot);
  frc::SmartDashboard::PutData(AutoModeName, &autoChooser);
}

void Autonomous::Periodic() {
  if (isRunning && mode.get() != nullptr) {
    mode->Periodic();
  }
}

void Autonomous::Start() {
  LOG_INFO(TELEMETRY_CATEGORY, "Start");

  const auto selectedMode = autoChooser.GetSelected();
  LOG_INFO(TELEMETRY_CATEGORY, fmt::format("Selected: '{}'", selectedMode));

  if (selectedMode == AutoDoNothing) {
    LOG_INFO(TELEMETRY_CATEGORY, "Do nothing auto");
    mode.reset(nullptr);
  }
  else if (selectedMode == AutoModeBasic) {
    LOG_INFO(TELEMETRY_CATEGORY, "Basic auto");
    mode.reset(new Rebels::AutoBasic());
  }
  else if (selectedMode == AutoModeShoot) {
    LOG_INFO(TELEMETRY_CATEGORY, "Shoot auto");
    mode.reset(new Rebels::AutoShoot());
  }
  else {
    LOG_WARNING(TELEMETRY_CATEGORY, "Unknown auto mode [!]");
    if (mode.get() != nullptr) {
      mode.reset(nullptr);
    }
  }

  isRunning = true;
}

void Autonomous::Stop() {
  LOG_INFO(TELEMETRY_CATEGORY, "Stop");

  isRunning = false;
  if (mode.get() != nullptr) {
    mode.reset(nullptr);
  }
}
