#pragma once
#include <ctre/Phoenix.h>
#include <frc/DoubleSolenoid.h>

#include "Constants.h"
#include "Settings.h"
#include "Singleton.h"
#include "StateMachine.h"

namespace Rebels {

  enum IntakeState {
    INTAKE_INIT,
    INTAKE_IDLE,
    INTAKE_EXTENDING,
    INTAKE_EXTENDED,
    INTAKE_ROLLING,
    INTAKE_RETRACTING,
  };

  const StateMachine<IntakeState>::StateStrings IntakeStrings = {
    "Init",
    "Idle",
    "Extending", // Extending out, not rolling
    "Extended",  // Extended, not rolling
    "Rolling",   // Extended and rolling (see 'reversed' for direction)
    "Retracting" // Retracting, not rolling
  };

  class Intake : public StateMachine<IntakeState> {
    DECLARE_SINGLETON(Intake)

  public:
    virtual ~Intake() = default;
    void Reset() { ForceState(INTAKE_INIT); }

  public:
    virtual void Update() override;
    virtual void BackgroundUpdate() override;

    // Actions
    void Extend();
    void Retract();
    void Grab();
    void Eject();

    bool IsRetracted() const;
    bool IsReversed() const;

  protected:
    virtual bool TransitionTo(IntakeState newState) override;

  private:
    Intake();
    void Init();
    void StopMotors();
    void SetRollerMotorPower(double power);
    void FireSolenoid(frc::DoubleSolenoid::Value value);

  private:
    double rollerMotorPower{ 0 };
    std::atomic_bool reversed = false;

#ifdef ENABLE_INTAKE
    VictorSPX rollerMotor{ INTAKE_ROLLER_MOTOR_ID };
#ifdef ENABLE_COMPRESSOR
    frc::DoubleSolenoid solenoid{ COMPRESSOR_MODULE_TYPE, INTAKE_FORWARD_PCM_ID, INTAKE_BACKWARD_PCM_ID };
#endif
#endif
  };
}  // namespace Rebels
