#pragma once
#include <ctre/Phoenix.h>
#include <ctre/phoenix/music/Orchestra.h>

#include "Constants.h"
#include "Settings.h"
#include "Singleton.h"
#include "StateMachine.h"

namespace Rebels {

  enum ShooterState {
    SHOOTER_INIT,
    SHOOTER_IDLE,
    SHOOTER_SHOOTING,
  };

  const StateMachine<ShooterState>::StateStrings ShooterStrings = {
    "Init",
    "Idle",
    "Shooting",
  };

  class Shooter : public StateMachine<ShooterState> {
    DECLARE_SINGLETON(Shooter)

  public:
    virtual ~Shooter() = default;
    void Reset() { ForceState(SHOOTER_INIT); }

  public:
    virtual void Update() override;
    virtual void BackgroundUpdate() override;

    // Actions
    void Start();
    void Stop();

    bool IsRunning() const;
    bool IsReady() const;

    void SetFlywheelTarget(double rpm);
    void SetTopRollerTarget(double rpm);

    void MakeSound();

  protected:
    virtual bool TransitionTo(ShooterState newState) override;

  private:
    Shooter();
    void Init();
    void InitMotor(TalonFX& motor, bool bInverted);
    void InitOrchestra();
    void SetMotorSpeedRpm(TalonFX& motor, const double rpm, const double kP, const double kI, const double kD, const double kF, const double ramp, const double relativeError);
    void SetShooterSpeedRpm(const double flywheelRPM, const double topRollerRPM);
    void PlayMusic();
    void StopMusic();
    bool IsMusicPlaying();

  private:
    double flywheelDesiredRpm{ 0 };
    double flywheelCurrentRpm{ 0 };
    double topRollerDesiredRpm{ 0 };
    double topRollerCurrentRpm{ 0 };

#ifdef ENABLE_SHOOTER
    TalonFX flywheelMotor{ SHOOTER_STAGE1_MOTOR_ID };
    TalonFX topRollerMotor{ SHOOTER_STAGE2_MOTOR_ID };
    Orchestra orchestra;
#endif
  };
}  // namespace Rebels


/*
Shooter State: Idle
At Speed : False
Flywheel RPM : 3000
Current Draw :
Target RPM:

TopRoller RPM: 4000
Current Draw :
Target RPM:
___________________
          [EditableToggle?]
Have this all in a table?
RPM Threshold : 50
{ {Group them if possible.}
  FlywheelP
  FlywheelI
  FlywheelD
  FlywheelF
  FlywheelLoopError
  FlywheelRamp
  FlywheelTargetRpm
  FlywheelThresholdRpm
}
{
  TopRollerP
  TopRollerI
  TopRollerD
  TopRollerF
  TopRollerLoopError
  TopRollerRamp
  TopRollerTargetRpm
  TopRollerThresholdRpm
}

*/