#include "Teleop.h"
#include "Settings.h"
#include "Telemetry.h"
#include "PointGuard.h"
#include "Climber.h"
#include "Limelight.h"

using namespace Rebels;

#define TELEMETRY_TELEOP "Teleop"

Teleop::Teleop() : led(LED_PWM_ID, LED_LENGTH) {}

void Teleop::Periodic() {
  XboxTeleop::Periodic();

  CheckShooter();
  CheckIntake();
  //CheckFeeder();
  CheckClimber();
  CheckAux();

  if (driverController.GetBackButtonReleased()) {
    PointGuard::Get()->Reset();
    Climber::Get()->Reset();
  }
}

void Teleop::CheckAux() {

  if (driverController.GetPOV() == -1)
    return;

  if (driverController.GetPOV() == 0) {
    if (Limelight::GetLedState() != LimelightLedState::On)
      Limelight::SetLedState(LimelightLedState::On);
    return;
  }

  if (driverController.GetPOV() == 180) {
    if (Limelight::GetLedState() != LimelightLedState::Off)
      Limelight::SetLedState(LimelightLedState::Off);
    return;
  }
}

void Teleop::CheckIntake() {
#ifdef ENABLE_INTAKE
  const auto pointGuard = PointGuard::Get();
  const auto intakeState = Intake::Get()->GetCurrentState();

  if (driverController.GetRightTriggerAxis() > 0.5) {
    pointGuard->StartIntake();
  }
  else if (intakeState != INTAKE_IDLE) {
    pointGuard->StopIntake();
  }

  if (driverController.GetRightBumperPressed()) {
    pointGuard->StartEjection();
  }
  if (driverController.GetRightBumperReleased()) {
    pointGuard->StopEjection();
  }
#endif
}

// void Teleop::CheckFeeder() {
// #ifdef ENABLE_FEEDER
// #endif
// }

void Teleop::CheckShooter() {
#ifdef ENABLE_SHOOTER
  const auto pointGuard = PointGuard::Get();
  const auto pointGuardState = pointGuard->GetCurrentState();

  if (driverController.GetLeftBumperPressed()) {
    LOG_DEBUG(TELEMETRY_TELEOP, "Left Bumper Pressed");
    pointGuard->StartShootingManual();
  }
  else if (driverController.GetLeftBumperReleased()) {
    LOG_DEBUG(TELEMETRY_TELEOP, "Left Bumper Released");
    pointGuard->StopShooting();
  }

  if (driverController.GetLeftTriggerAxis() > 0.5 && pointGuardState != PG_SHOOTING_VISION) {
    pointGuard->StartShootingVision();
  }
  if (driverController.GetLeftTriggerAxis() < 0.5 && pointGuardState == PG_SHOOTING_VISION) {
    pointGuard->StopShooting();
  }

  if (driverController.GetStartButtonPressed()) {
    Shooter::Get()->MakeSound();
  }
#endif
}

void Teleop::CheckClimber() {
#ifdef ENABLE_CLIMBER
  const auto climber = Climber::Get();
  const auto climberState = climber->GetCurrentState();

  if (driverController.GetAButtonReleased()) {
    if (climberState == CLIMBER_IDLE || climberState == CLIMBER_RETRACTED)
      climber->Extend();
    if (climberState == CLIMBER_EXTENDED)
      climber->Retract();
  }

  if (driverController.GetBButtonReleased()) {
    if (climberState == CLIMBER_RETRACTED || climberState == CLIMBER_RETRACTED_NEXT_BAR)
      climber->NextBarExtend();
    if (climberState == CLIMBER_EXTENDED_NEXT_BAR)
      climber->NextBarRetract();
  }

  // if (driverController.GetXButtonReleased()) {
  //   climber->ReversePneumatics();
  // }

  if (driverController.GetYButtonReleased()) {
    climber->AutoHome();
  }

#endif
}